/*
For information on this software or to report bugs please visit http://www.ranton.org
or email ranton.org

Copyright (c) 2002, Richard Neal Anton
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

*	Redistributions of source code must retain the above copyright notice, this list
	of conditions and the following disclaimer. 

*	Redistributions in binary form must reproduce the above copyright notice, this list
	of conditions and the following disclaimer in the documentation and/or other materials
	provided with the distribution. 

*	The name of the copyright holder may not be used to endorse or promote products 
	derived from this software without specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <sys/sockio.h>

#include <netinet/in.h>
#include <arpa/inet.h>


// returns errno type value, 0 on success
int GetInterfaceInfo(int sockfd, char *name, int *outMtu, int *outMetric);


	
int main(int argc, char **argv)
{
	// print the name and MTU of all the network interfaces
	struct ifreq *ifrp;
	int sockfd;
	int total, remaining, current;
	struct ifconf ifc;
	struct sockaddr_in *addr;
#define MAX_IFS 32
	char buf[sizeof(struct ifreq)*MAX_IFS];
	int err;
	int metric,mtu;
//	name = argv[1];
	
	// we need a socket to call ioctl() on
	sockfd = socket(PF_INET,SOCK_DGRAM,0);
	if( sockfd < 0 )
		perror("socket");
	
	ifc.ifc_len = sizeof( buf );
	ifc.ifc_buf = (caddr_t)buf;
	
	if (ioctl(sockfd, SIOCGIFCONF, (caddr_t)&ifc) < 0)
    	perror("ioctl (SIOCGIFCONF)");
	
	remaining = total = ifc.ifc_len;
	ifrp = ifc.ifc_req;
	while( remaining )
	{
		if( ifrp->ifr_addr.sa_family == AF_INET )
		{
			err = GetInterfaceInfo(sockfd, ifrp->ifr_name,&mtu,&metric);
			if( err )
			{
				errno = err;
				perror("ioctl");
				continue;
			}
			addr = (struct sockaddr_in *)&(ifrp->ifr_addr);
			printf("%s addr=%s mtu=%d metric=%d\n",
					ifrp->ifr_name, 
					inet_ntoa(addr->sin_addr),
					mtu,
					metric
					);
		}
		
		
		current = ifrp->ifr_addr.sa_len + IFNAMSIZ;
		ifrp = (struct ifreq *)( ((char *)ifrp)+current );
		remaining -= current;
	}
			
	// struct ifreq ioctl's
	// SIOCSIFFLAGS - get interface flags
	// SIOCGIFMETRIC - get interface metric
	
	// sturct ifconf ioctl's
	// SIOCGIFCONF - get interface configuration list
	
	// SIOCSIFMTU
	// SIOCGIFMTU
	// SIOCGIFSTATUS
	// SIOCDIFADDR
	// SIOCAIFADDR
	
	
	(void)close(sockfd);
	
	return 0;
}


/*
 * Print the status of the interface.  If an address family was
 * specified, show it and it only; otherwise, show them all.
void
status(afp, addrcount, sdl, ifm, ifam)
        const struct afswtch *afp;
        int addrcount;
        struct  sockaddr_dl *sdl;
        struct if_msghdr *ifm;
        struct ifa_msghdr *ifam;
{
        const struct afswtch *p = NULL;
        struct  rt_addrinfo info;
        int allfamilies, s;
        struct ifstat ifs;

        if (afp == NULL) {
                allfamilies = 1;
                afp = &afs[0];
        } else
                allfamilies = 0;

        ifr.ifr_addr.sa_family = afp->af_af == AF_LINK ? AF_INET : afp->af_af;
        strncpy(ifr.ifr_name, name, sizeof ifr.ifr_name);

        if ((s = socket(ifr.ifr_addr.sa_family, SOCK_DGRAM, 0)) < 0)
                err(1, "socket");

		if (ioctl(s, SIOCGIFMETRIC, (caddr_t)&ifr) < 0)
                warn("ioctl (SIOCGIFMETRIC)");
        else
                metric = ifr.ifr_metric;

        if (ioctl(s, SIOCGIFMTU, (caddr_t)&ifr) < 0)
                warn("ioctl (SIOCGIFMTU)");
        else
                mtu = ifr.ifr_mtu;

        printf("%s: ", name);
        printb("flags", flags, IFFBITS);
        if (metric)
                printf(" metric %d", metric);
        if (mtu)
                printf(" mtu %d", mtu);
        putchar('\n');
*/
		
		
		
// returns errno type value, 0 on success
int GetInterfaceInfo(int sockfd, char *name, int *outMtu, int *outMetric)
{
	struct ifreq ifr;

	ifr.ifr_addr.sa_family = AF_INET;
	strncpy(ifr.ifr_name, name, sizeof(ifr.ifr_name));

	if( outMetric )
	{
		if (ioctl(sockfd, SIOCGIFMETRIC, (caddr_t)&ifr) < 0)
			return errno;

		*outMetric = ifr.ifr_metric;
	}	
	
	if( outMtu )
	{
		if (ioctl(sockfd, SIOCGIFMTU, (caddr_t)&ifr) < 0)
			return errno;
		
		*outMtu = ifr.ifr_mtu;
	}

	return 0;
}
